
proc tstr {args} {
   foreach s $args {
      if {[string length $s] == 0} { return 0 }
   }
   return 1
}

# Hop data is as follows:
#
# $Hop() is a list containing the idx values from H1 .. H8
# The Hop(x) array contains
#       Hop($idx:amount) - amount of each hop in ounces (float)
#       Hop($idx:variety) - name of the hop
#       Hop($idx:aa) - %alpha for the hop
#       Hop($idx:time) - boil time for the hop
#       Hop($idx:ibu) - IBUs for the hop addition
#       Hop($idx:percent) - percent of total IBUs
#

proc calc_hops {{reverse 0}} {
   global Hop batchsize boilvol OGSG totalibus display_ibus hopmethod

   set totalibus 0
   foreach idx $Hop() {
      set name  $Hop($idx:variety)
      set alpha $Hop($idx:aa)
      set time  $Hop($idx:time)
      if {[streq $time "fwh"] || [streq $time "FWH"]} { set time 60 }
      if {[streq $time "eob"] || [streq $time "EOB"]} { set time 0 }
       
      if {!$reverse} {
         set amt   $Hop($idx:amount)
         if {[tstr $name $amt $alpha $time]} {
            set ibu [${hopmethod}_ibu $amt $alpha $time \
                     $batchsize $boilvol $OGSG]
            set Hop($idx:ibu) [format %.1f [expr {int(round(10.0*$ibu))/10.0}]]
            set totalibus [expr {(1.00*$ibu+$totalibus)}]
         } else {
            set Hop($idx:ibu) ""
            set Hop($idx:percent) ""
         }
      } else {
         set ibu   $Hop($idx:ibu)
         if {[tstr $name $ibu $alpha $time]} {
            set amt [${hopmethod}_amt $ibu $alpha $time \
                     $batchsize $boilvol $OGSG]
            set Hop($idx:amount) [format %.1f [expr {int(round(10.0*$amt))/10.0}]]
            set totalibus [expr {(1.00*$ibu+$totalibus)}]
         } else {
            set Hop($idx:amount) ""
            set Hop($idx:percent) ""
         }
      }

   }
   foreach idx $Hop() {
      if {[strlen $Hop($idx:ibu)]} {
         set pc [expr {round(100.00 * ($Hop($idx:ibu)/$totalibus))}]
         set Hop($idx:percent) $pc
      }
   }
   set display_ibus [format %.1f $totalibus]
   check_bjcp
}

set srmchart(0) #FFFFFF  ; set srmchart(15) #A04F2A ; set srmchart(30) #5E1C16
set srmchart(1) #F0EEBC  ; set srmchart(16) #9C4A28 ; set srmchart(31) #5A1915
set srmchart(2) #E9D67D  ; set srmchart(17) #984626 ; set srmchart(32) #571614
set srmchart(3) #DBBE57  ; set srmchart(18) #934225 ; set srmchart(33) #531313
set srmchart(4) #CCA532  ; set srmchart(19) #8F3D25 ; set srmchart(34) #201113
set srmchart(5) #D5A435  ; set srmchart(20) #8A3924 ; set srmchart(35) #4D1113
set srmchart(6) #CF9631  ; set srmchart(21) #863524 ; set srmchart(36) #4A1113
set srmchart(7) #C8882D  ; set srmchart(22) #813124 ; set srmchart(37) #461113
set srmchart(8) #C27E2F  ; set srmchart(23) #7C2E21 ; set srmchart(38) #431113
set srmchart(9) #BB7531  ; set srmchart(24) #772C1F ; set srmchart(39) #401113
set srmchart(10) #B66E30 ; set srmchart(25) #72291D ; set srmchart(40) #3D1113
set srmchart(11) #B1662F ; set srmchart(26) #6D271B ; set srmchart(41) #3A1113
set srmchart(12) #AC5F2E ; set srmchart(27) #682519 ; set srmchart(42) #371113
set srmchart(13) #A7582E ; set srmchart(28) #642218 ; set srmchart(43) #341113
set srmchart(14) #A3532C ; set srmchart(29) #611F17 ; set srmchart(44) #311214

# Calculate the partial color contribution for individual grains from
# the grain's color (degL), weight (pounds), and batch size (gallons):

proc partialcolor {color weight batchsize} {

   if {$batchsize > 0} {
      set partial [expr {($weight*$color)/$batchsize}]
   } else {
      set partial 0
   }
   return $partial
}


# check to make sure all args have been filled in before doing the math

proc null {args} {
   foreach str $args {
      if {[string length $str] == 0} { return 1 }
   }
   return 0
}

proc oz2lbs {oz} { return [expr {$oz/16.0}] }


# This is ibucalcs.tcl.  It has the procs that do the math for Rager and
# Tinseth.  The math for Rager has been checked against BLH's math, and
# finally worked out correctly.  Details:  BLH (or my recipes in BLH, I'm
# not sure) uses boil volume == final volume.  That's the first reason
# the numbers may appear to be off.  Second, BLH subtracts 10% from the
# bitterness if a hop bag is used.  It also adjusts per yeast used, but
# since I never entered Brewers Resource's yeast data into BLH, I never
# used that bit...so I'm not trying to match that up against my logs.
#
# To calculate the amount of hops for a given IBU using a known %alpha,
# OG, final volume (gallons), boil volume (gallons), and boil time:
#
# RAGER:
#    rager_amt desired_ibu %alpha boil_time final_vol boil_vol orig_grav
#
# TINSETH:
#    tinseth_amt desired_ibu %alpha boil_time final_vol boil_vol orig_grav
#
# To calculate the reverse, i.e., the IBUs for a given hop addition:
#
# RAGER:
#    rager_ibu oz_hops %alpha boil_time final_vol boil_vol orig_grav
#
# TINSETH:
#    tinseth_ibu oz_hops %alpha boil_time final_vol boil_vol orig_grav

# This does the bulk of the work for Rager hop calculations
#
proc rager_core {alpha time tot_vol boil_vol grav} {
   set utila [expr {13.86 * tanh(($time - 31.32) / 18.27)}]
   set util [expr {0.01 * (18.11 + $utila)}]

   set ga 0 
   if {$grav > 1.050} {
      set bgrav [expr {1 + ($grav - 1) * ($tot_vol/$boil_vol)}]
      set ga [expr {(($bgrav - 1.050) / 0.2)}]
   }

   set ibua [expr {$util * $alpha * 7489 * .01}]
   set ibub [expr {$tot_vol * (1 + $ga)}]
   return [expr {$ibua / $ibub}]
}

# rager_ibu takes the following args:
# amount in oz (amt), % alpha acids (alpha), boil time (time), final
# volume (tot_vol), boil_volume, and final gravity (grav), and returns
# the bitterness in IBUs for that hop addition using Rager.
#
proc rager_ibu {amt alpha time tot_vol boil_vol grav} {
   if {$time == 0} { return 0 }
   if {[null $amt $alpha $time $tot_vol $boil_vol $grav]} { return 0 }

   set ibutmp [rager_core $alpha $time $tot_vol $boil_vol $grav]
   set ibus [expr {$amt * $ibutmp}]
}


# rager_amt takes basically the same args, except that instead of the
# amount in oz, it takes the desired IBUs for the hop addition and
# returns the amount in oz.  The other args are the same as rager_ibu.
#
proc rager_amt {ibu alpha time tot_vol boil_vol grav} {
   if {[null $ibu $alpha $time $tot_vol $boil_vol $grav]} { return 0 }

   set ibutmp [rager_core $alpha $time $tot_vol $boil_vol $grav]
   set amt  [expr {$ibu / $ibutmp}]
}


proc tinseth_core {alpha time tot_vol bl_vol grav} {
   set adj2 [expr {(($grav - 1) * ($tot_vol/$bl_vol))}]
   set adj [expr {1.65 * pow(1.25e-4,$adj2)}]
   return [expr {(1 - exp(-0.04*$time))/4.15 * 74.90 * $alpha * $adj/$tot_vol}]
}

proc tinseth_ibu {amt alpha time tot_vol bl_vol grav} {
   if {$time == 0} { return 0 }
   if {[null $amt $alpha $time $tot_vol $bl_vol $grav]} { return 0 }

   set ibutmp [tinseth_core $alpha $time $tot_vol $bl_vol $grav]
   set ibus [expr {$ibutmp * $amt}]
}

proc tinseth_amt {ibu alpha time tot_vol bl_vol grav} {
   if {[null $ibu $alpha $time $tot_vol $bl_vol $grav]} { return 0 }

   set ibutmp [tinseth_core $alpha $time $tot_vol $bl_vol $grav]
   set amt [expr {$ibu/$ibutmp}]
}


proc getvolsg {} {
   global batchsize OGSG diluvals preboilvol preboilsg
   
   set postsg $OGSG
   set postvol $batchsize
   set prevol [expr {$preboilvol * 0.97}]
   set presg $preboilsg

   if {$presg <= 0 || $prevol <= 0} { return }
   set adjpresg [expr {($presg - 1.000) * 1000.0000}]

   if {$postvol > 0} {
      set l1tmp [expr {1 + (double($prevol/$postvol) * $adjpresg)/1000.0000}]
      set dilu1 "Post-boil:  OG at $postvol gal (chilled):  [format %.3f $l1tmp]"
   }

   if {$postsg > 0} {
      set adjpostsg [expr {($postsg - 1.000) * 1000.0000}]
      set l2tmp [expr {(double($adjpresg/$adjpostsg) * $prevol)/0.96}]
      set dilu2 "[format %1.2f $l2tmp] gal (vol at boil temp) for [format %.3f $postsg] OG"
   }
   .c.dilution configure -state normal
   set diluvals "$dilu1 // $dilu2"
}


# percent alcohol (v/v)
#
# Revised per BSI (Brewing Science International) Brewers Lab Handbook
#
# OE = original extract,  Plato
# AE = apparent extract,  Plato
# % alcohol by weight = 0.42 (OE - AE)
# % abv = (% alcohol by weight x specific gravity of beer) / 0.791

proc get_abv {} {
   global ogactual fgactual

   set OE [SG_to_Plato $ogactual]
   set AE [SG_to_Plato $fgactual]
   return [format %.1f [expr {((0.42*($OE - $AE))/0.791)}]]
}


